@@ -170,7 +170,7 @@ static void save_last_error(struct accfg_device *device, struct accfg_wq *wq,
 }
 
 static int accfg_set_param(struct accfg_ctx *ctx, int dfd, char *name,
-		void *buf, int len)
+		const void *buf, int len)
 {
 	int fd = openat(dfd, name, O_RDWR);
 	int n;
@@ -579,10 +579,10 @@ static void *add_device(void *parent, int id, const char *ctl_base,
 	device->gencap = accfg_get_param_unsigned_llong(ctx, dfd, "gen_cap");
 	device->configurable = accfg_get_param_unsigned_llong(ctx, dfd,
 			"configurable");
-	device->pasid_enabled = accfg_get_param_str(ctx, dfd,
-			"pasid_enabled");
+	device->pasid_enabled = accfg_get_param_long(ctx, dfd, "pasid_enabled");
 	device->max_read_buffers = accfg_get_param_long(ctx, dfd, "max_read_buffers");
 	device->read_buffer_limit = accfg_get_param_long(ctx, dfd, "read_buffer_limit");
+	device->event_log_size = accfg_get_param_long(ctx, dfd, "event_log_size");
 	device->cdev_major = accfg_get_param_long(ctx, dfd, "cdev_major");
 	device->version = accfg_get_param_unsigned_llong(ctx, dfd, "version");
 	device->device_path = realpath(ctl_base, NULL);
@@ -708,6 +708,7 @@ static void *add_wq(void *parent, int id, const char *wq_base,
 	wq->max_batch_size =  accfg_get_param_long(ctx, dfd, "max_batch_size");
 	wq->max_transfer_size =  accfg_get_param_long(ctx, dfd, "max_transfer_size");
 	wq->ats_disable = accfg_get_param_long(ctx, dfd, "ats_disable");
+	wq->prs_disable = accfg_get_param_long(ctx, dfd, "prs_disable");
 
 	wq_parse_type(wq, wq_type);
 	free(wq_type);
@@ -794,6 +795,10 @@ static void *add_group(void *parent, int id, const char *group_base,
 			"traffic_class_a");
 	group->traffic_class_b = accfg_get_param_long(ctx, dfd,
 			"traffic_class_b");
+	group->desc_progress_limit = accfg_get_param_long(ctx, dfd,
+			"desc_progress_limit");
+	group->batch_progress_limit = accfg_get_param_long(ctx, dfd,
+			"batch_progress_limit");
 
 	close(dfd);
 	group->group_buf = calloc(1, strlen(group_base) + MAX_BUF_LEN);
@@ -1091,13 +1096,18 @@ ACCFG_EXPORT int accfg_device_get_op_cap(struct accfg_device *device,
 		return -errno;
 	oc = accfg_get_param_str(ctx, dfd, "op_cap");
 	close(dfd);
-	rc = sscanf(oc, "%" SCNx64 " %" SCNx64 " %" SCNx64 " %" SCNx64,
+	if (!oc)
+		return -EIO;
+	rc = sscanf(oc, "%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32
+			",%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32,
 			&op_cap->bits[0], &op_cap->bits[1],
-			&op_cap->bits[2], &op_cap->bits[3]);
+			&op_cap->bits[2], &op_cap->bits[3],
+			&op_cap->bits[4], &op_cap->bits[5],
+			&op_cap->bits[6], &op_cap->bits[7]);
 
 	free(oc);
 
-	if (rc != 4)
+	if (rc != 8)
 		return errno ? -errno : -EIO;
 
 	return 0;
@@ -1108,6 +1118,35 @@ ACCFG_EXPORT uint64_t accfg_device_get_gen_cap(struct accfg_device *device)
 	return device->gencap;
 }
 
+ACCFG_EXPORT int accfg_device_get_iaa_cap(struct accfg_device *device,
+		uint64_t *iaa_cap)
+{
+	int fd;
+	int rc;
+	struct accfg_ctx *ctx;
+	char *buf;
+
+	if (!device)
+		return -EINVAL;
+
+	ctx = accfg_device_get_ctx(device);
+	fd = open(device->device_path, O_PATH);
+	if (fd < 0)
+		return -errno;
+	buf = accfg_get_param_str(ctx, fd, "iaa_cap");
+	close(fd);
+	if (!buf)
+		return -EIO;
+
+	errno = 0;
+	*iaa_cap = strtoull(buf, NULL, 0);
+	rc = -errno;
+
+	free(buf);
+
+	return rc;
+}
+
 ACCFG_EXPORT unsigned int accfg_device_get_configurable(
 		struct accfg_device *device)
 {
@@ -1137,13 +1176,16 @@ ACCFG_EXPORT int accfg_device_get_errors(struct accfg_device *device,
 		return -errno;
 	read_error = accfg_get_param_str(ctx, dfd, "errors");
 	close(dfd);
-	rc = sscanf(read_error, "%" SCNx64 " %" SCNx64 " %" SCNx64 " %" SCNx64,
+	rc = sscanf(read_error, "%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32
+			",%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32,
 			&error->val[0], &error->val[1],
-			&error->val[2], &error->val[3]);
+			&error->val[2], &error->val[3],
+			&error->val[4], &error->val[5],
+			&error->val[6], &error->val[7]);
 
 	free(read_error);
 
-	if (rc != 4)
+	if (rc != 8)
 		return errno ? -errno : -EIO;
 
 	return 0;
@@ -1197,6 +1239,11 @@ ACCFG_EXPORT unsigned int accfg_device_get_read_buffer_limit(
 	return device->read_buffer_limit;
 }
 
+ACCFG_EXPORT int accfg_device_get_event_log_size(struct accfg_device *device)
+{
+	return device->event_log_size;
+}
+
 ACCFG_EXPORT unsigned int accfg_device_get_cdev_major(
 		struct accfg_device *device)
 {
@@ -1287,6 +1334,46 @@ ACCFG_EXPORT int accfg_device_set_read_buffer_limit(struct accfg_device *dev, in
 	return 0;
 }
 
+ACCFG_EXPORT int accfg_device_set_event_log_size(struct accfg_device *dev, int val)
+{
+	struct accfg_ctx *ctx;
+	char *path;
+	char buf[SYSFS_ATTR_SIZE];
+
+	if (!dev)
+		return -EINVAL;
+
+	path = dev->device_buf;
+	ctx = accfg_device_get_ctx(dev);
+
+	if (sprintf(path, "%s/event_log_size", dev->device_path) >=
+			(int)dev->buf_len) {
+		err(ctx, "%s; buf len exceeded.\n",
+				accfg_device_get_devname(dev));
+		return -errno;
+	}
+
+	if (access(path, F_OK))
+		return 0;
+
+	if (sprintf(buf, "%d", val) < 0) {
+		err(ctx, "%s: sprintf to buf failed: %s\n",
+				accfg_device_get_devname(dev), strerror(errno));
+		return -errno;
+	}
+
+	if (sysfs_write_attr(ctx, path, buf) < 0) {
+		err(ctx, "%s: write failed: %s\n",
+				accfg_device_get_devname(dev), strerror(errno));
+		save_last_error(dev, NULL, NULL, NULL);
+		return -errno;
+	}
+
+	dev->event_log_size = val;
+
+	return 0;
+}
+
 ACCFG_EXPORT int accfg_device_is_active(struct accfg_device *device)
 {
 	struct accfg_ctx *ctx;
@@ -1667,6 +1754,8 @@ accfg_group_set_field(group, val, read_buffers_allowed)
 accfg_group_set_field(group, val, use_read_buffer_limit)
 accfg_group_set_field(group, val, traffic_class_a)
 accfg_group_set_field(group, val, traffic_class_b)
+accfg_group_set_field(group, val, desc_progress_limit)
+accfg_group_set_field(group, val, batch_progress_limit)
 
 #define accfg_group_get_field(group, field) \
 ACCFG_EXPORT int accfg_group_get_##field( \
@@ -1680,6 +1769,8 @@ accfg_group_get_field(group, read_buffers_allowed);
 accfg_group_get_field(group, use_read_buffer_limit);
 accfg_group_get_field(group, traffic_class_a);
 accfg_group_get_field(group, traffic_class_b);
+accfg_group_get_field(group, desc_progress_limit);
+accfg_group_get_field(group, batch_progress_limit);
 
 static void wqs_init(struct accfg_device *device)
 {
@@ -1866,6 +1957,84 @@ ACCFG_EXPORT int accfg_wq_priority_boundary(struct accfg_wq *wq)
 	return 0;
 }
 
+ACCFG_EXPORT int accfg_wq_get_op_config(struct accfg_wq *wq,
+		struct accfg_op_config *op_config)
+{
+	char *oc;
+	int dfd;
+	int rc;
+	struct accfg_ctx *ctx;
+
+	if (!wq || !op_config)
+		return -EINVAL;
+
+	ctx = accfg_wq_get_ctx(wq);
+	dfd = open(wq->wq_path, O_PATH);
+	if (dfd < 0)
+		return -errno;
+	oc = accfg_get_param_str(ctx, dfd, "op_config");
+	close(dfd);
+	if (!oc)
+		return -EIO;
+	rc = sscanf(oc, "%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32
+			",%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32,
+			&op_config->bits[0], &op_config->bits[1],
+			&op_config->bits[2], &op_config->bits[3],
+			&op_config->bits[4], &op_config->bits[5],
+			&op_config->bits[6], &op_config->bits[7]);
+
+	free(oc);
+
+	if (rc != 8)
+		return -EIO;
+
+	return 0;
+}
+
+ACCFG_EXPORT int accfg_wq_set_op_config(struct accfg_wq *wq,
+		struct accfg_op_config *op_config)
+{
+	char oc[MAX_PARAM_LEN];
+
+	if (!wq || !op_config)
+		return -EINVAL;
+
+	snprintf(oc, MAX_PARAM_LEN,
+			"%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32
+			",%" SCNx32 ",%" SCNx32 ",%" SCNx32 ",%" SCNx32,
+			op_config->bits[0], op_config->bits[1],
+			op_config->bits[2], op_config->bits[3],
+			op_config->bits[4], op_config->bits[5],
+			op_config->bits[6], op_config->bits[7]);
+
+	return accfg_wq_set_op_config_str(wq, oc);
+}
+
+ACCFG_EXPORT int accfg_wq_set_op_config_str(struct accfg_wq *wq,
+		const char *op_config)
+{
+	struct accfg_ctx *ctx;
+	int dfd;
+	int rc;
+
+	if (!wq || !op_config)
+		return -EINVAL;
+
+	ctx = accfg_wq_get_ctx(wq);
+	dfd = open(wq->wq_path, O_PATH);
+	if (dfd < 0)
+		return -errno;
+
+	rc = accfg_set_param(ctx, dfd, "op_config", op_config,
+			strlen(op_config));
+
+	close(dfd);
+	if (rc)
+		return -EIO;
+
+	return 0;
+}
+
 static int accfg_wq_retrieve_cdev_minor(struct accfg_wq *wq)
 {
 	int dfd;
@@ -2129,6 +2298,7 @@ accfg_wq_set_field(wq, val, block_on_fault)
 accfg_wq_set_field(wq, val, threshold)
 accfg_wq_set_field(wq, val, max_batch_size)
 accfg_wq_set_field(wq, val, ats_disable)
+accfg_wq_set_field(wq, val, prs_disable)
 
 #define accfg_wq_set_long_field(wq, val, field) \
 ACCFG_EXPORT int accfg_wq_set_##field( \
@@ -2264,6 +2434,7 @@ accfg_wq_get_field(wq, threshold)
 accfg_wq_get_field(wq, group_id)
 accfg_wq_get_field(wq, block_on_fault)
 accfg_wq_get_field(wq, ats_disable)
+accfg_wq_get_field(wq, prs_disable)
 
 ACCFG_EXPORT int accfg_wq_set_mode(struct accfg_wq *wq,
 		enum accfg_wq_mode wq_mode)
