@@ -11,6 +11,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/mman.h>
+#include <sys/user.h>
 #include <linux/vfio.h>
 #include <accfg/libaccel_config.h>
 #include <accfg/idxd.h>
@@ -51,7 +52,7 @@ unsigned long get_blks(unsigned long xfer_size)
 
 int init_memcpy(struct task *tsk, int tflags, int opcode, unsigned long xfer_size)
 {
-	unsigned long force_align = ADDR_ALIGNMENT;
+	unsigned long force_align = PAGE_SIZE;
 
 	tsk->pattern = 0x0123456789abcdef;
 	tsk->pattern2 = 0xfedcba9876543210;
@@ -144,12 +145,12 @@ int init_dualcast(struct task *tsk, int tflags, int opcode, unsigned long xfer_s
 		return -ENOMEM;
 	memset_pattern(tsk->src1, tsk->pattern, xfer_size);
 
-	tsk->dst1 = aligned_alloc(1 << 12, xfer_size);
+	tsk->dst1 = aligned_alloc(PAGE_SIZE, xfer_size);
 	if (!tsk->dst1)
 		return -ENOMEM;
 	memset_pattern(tsk->dst1, tsk->pattern2, xfer_size);
 
-	tsk->dst2 = aligned_alloc(1 << 12, xfer_size);
+	tsk->dst2 = aligned_alloc(PAGE_SIZE, xfer_size);
 	if (!tsk->dst2)
 		return -ENOMEM;
 	memset_pattern(tsk->dst2, tsk->pattern2, xfer_size);
@@ -157,6 +158,21 @@ int init_dualcast(struct task *tsk, int tflags, int opcode, unsigned long xfer_s
 	return ACCTEST_STATUS_OK;
 }
 
+int init_transl_fetch(struct task *tsk, int tflags, int opcode, unsigned long xfer_size)
+{
+	unsigned long force_align = ADDR_ALIGNMENT;
+
+	tsk->opcode = opcode;
+	tsk->test_flags = tflags;
+	tsk->xfer_size = xfer_size;
+
+	tsk->src1 = aligned_alloc(force_align, tsk->xfer_size);
+	if (!tsk->src1)
+		return -ENOMEM;
+
+	return ACCTEST_STATUS_OK;
+}
+
 int init_cr_delta(struct task *tsk, int tflags, int opcode, unsigned long xfer_size)
 {
 	unsigned long force_align = ADDR_ALIGNMENT;
@@ -483,6 +499,10 @@ int init_task(struct task *tsk, int tflags, int opcode,
 		rc = init_dualcast(tsk, tflags, opcode, xfer_size);
 		break;
 
+	case DSA_OPCODE_TRANSL_FETCH:
+		rc = init_transl_fetch(tsk, tflags, opcode, xfer_size);
+		break;
+
 	case DSA_OPCODE_AP_DELTA:
 	case DSA_OPCODE_CR_DELTA:
 		rc = init_cr_delta(tsk, tflags, opcode, xfer_size);
@@ -501,6 +521,7 @@ int init_task(struct task *tsk, int tflags, int opcode,
 		break;
 
 	case DSA_OPCODE_DIF_INS:
+	case DSA_OPCODE_DIX_GEN:
 		rc = init_dif_ins(tsk, tflags, opcode, xfer_size);
 		break;
 
@@ -533,6 +554,8 @@ int alloc_batch_task(struct acctest_context *ctx, unsigned int task_num, int num
 	struct btask_node *btsk_node;
 	struct batch_task *btsk;
 	int cnt = 0;
+	int prot = PROT_READ | PROT_WRITE;
+	int mmap_flags = MAP_ANONYMOUS | MAP_PRIVATE;
 
 	if (!ctx->is_batch) {
 		err("%s is valid only if 'is_batch' is enabled", __func__);
@@ -563,18 +586,34 @@ int alloc_batch_task(struct acctest_context *ctx, unsigned int task_num, int num
 			return -ENOMEM;
 		memset(btsk->sub_tasks, 0, task_num * sizeof(struct task));
 
-		btsk->sub_descs = aligned_alloc(64, task_num * sizeof(struct hw_desc));
-		if (!btsk->sub_descs)
-			return -ENOMEM;
-		memset(btsk->sub_descs, 0, task_num * sizeof(struct hw_desc));
+		if (ctx->is_evl_test) {
+			btsk->sub_descs = mmap(NULL, PAGE_ALIGN(PAGE_SIZE +
+					       task_num * sizeof(struct hw_desc)),
+					       prot, mmap_flags, -1, 0);
+			if (!btsk->sub_descs)
+				return -ENOMEM;
+			memset(btsk->sub_descs, 0, PAGE_ALIGN(PAGE_SIZE +
+			       task_num * sizeof(struct hw_desc)));
 
-		/* To be compatible with IAX, completion record need to be 64-byte aligned */
-		btsk->sub_comps =
-			aligned_alloc(64, task_num * sizeof(struct completion_record));
-		if (!btsk->sub_comps)
-			return -ENOMEM;
-		memset(btsk->sub_comps, 0,
-		       task_num * sizeof(struct completion_record));
+			btsk->sub_comps = aligned_alloc(PAGE_SIZE, task_num * PAGE_SIZE);
+			if (!btsk->sub_comps)
+				return -ENOMEM;
+			memset(btsk->sub_comps, 0,
+			       task_num * PAGE_SIZE);
+		} else	{
+			btsk->sub_descs = aligned_alloc(64, task_num * sizeof(struct hw_desc));
+			if (!btsk->sub_descs)
+				return -ENOMEM;
+			memset(btsk->sub_descs, 0, task_num * sizeof(struct hw_desc));
+
+			/* IAX completion record need to be 64-byte aligned */
+			btsk->sub_comps =
+				aligned_alloc(64, task_num * sizeof(struct completion_record));
+			if (!btsk->sub_comps)
+				return -ENOMEM;
+			memset(btsk->sub_comps, 0,
+			       task_num * sizeof(struct completion_record));
+		}
 
 		dbg("batch task allocated %#lx, ctask %#lx, sub_tasks %#lx\n",
 		    btsk, btsk->core_task, btsk->sub_tasks);
@@ -597,7 +636,11 @@ int init_batch_task(struct batch_task *btsk, int task_num, int tflags,
 
 	for (i = 0; i < task_num; i++) {
 		btsk->sub_tasks[i].desc = &btsk->sub_descs[i];
-		btsk->sub_tasks[i].comp = &btsk->sub_comps[i];
+		if (btsk->edl)
+			btsk->sub_tasks[i].comp = &btsk->sub_comps[(PAGE_SIZE * i) /
+				sizeof(struct completion_record)];
+		else
+			btsk->sub_tasks[i].comp = &btsk->sub_comps[i];
 		btsk->sub_tasks[i].dflags = dflags;
 		rc = init_task(&btsk->sub_tasks[i], tflags, opcode, xfer_size);
 		if (rc != ACCTEST_STATUS_OK) {
@@ -661,12 +704,22 @@ int dsa_wait_batch(struct batch_task *btsk, struct acctest_context *ctx)
 
 	info("wait batch\n");
 
+again:
 	rc = acctest_wait_on_desc_timeout(ctsk->comp, ctx, ms_timeout);
 	if (rc < 0) {
 		err("batch desc timeout\n");
 		return ACCTEST_STATUS_TIMEOUT;
 	}
 
+	if (stat_val(ctsk->comp->status) == DSA_COMP_BATCH_PAGE_FAULT) {
+		struct hw_desc *hw = ctsk->desc;
+
+		if (hw->desc_count - ctsk->comp->descs_completed >= 2) {
+			dsa_reprep_batch(btsk, ctx);
+			goto again;
+		}
+	}
+
 	dump_sub_compl_rec(btsk, ctx->compl_size);
 	return ACCTEST_STATUS_OK;
 }
@@ -730,8 +783,9 @@ again:
 	}
 
 	/* re-submit if PAGE_FAULT reported by HW && BOF is off */
-	if (stat_val(comp->status) == DSA_COMP_PAGE_FAULT_NOBOF &&
-	    !(desc->flags & IDXD_OP_FLAG_BOF)) {
+	if ((stat_val(comp->status) == DSA_COMP_PAGE_FAULT_NOBOF &&
+	     !(desc->flags & IDXD_OP_FLAG_BOF)) ||
+		(stat_val(comp->status) == DSA_COMP_CRA_XLAT)) {
 		dsa_reprep_memcpy(ctx, tsk);
 		goto again;
 	}
@@ -756,6 +810,9 @@ int dsa_memcpy_multi_task_nodes(struct acctest_context *ctx)
 	info("Submitted all memcpy jobs\n");
 	tsk_node = ctx->multi_task_node;
 	while (tsk_node) {
+		if (tsk_node->tsk->test_flags & TEST_FLAGS_CPFLT)
+			madvise(tsk_node->tsk->comp, 4096, MADV_DONTNEED);
+
 		acctest_desc_submit(ctx, tsk_node->tsk->desc);
 		tsk_node = tsk_node->next;
 	}
@@ -999,6 +1056,53 @@ int dsa_dualcast_multi_task_nodes(struct acctest_context *ctx)
 	return ret;
 }
 
+int dsa_wait_transl_fetch(struct acctest_context *ctx, struct task *tsk)
+{
+	struct completion_record *comp = tsk->comp;
+	int rc;
+
+	rc = acctest_wait_on_desc_timeout(comp, ctx, ms_timeout);
+	if (rc < 0) {
+		err("transl fetch desc timeout\n");
+		return ACCTEST_STATUS_TIMEOUT;
+	}
+
+	return ACCTEST_STATUS_OK;
+}
+
+int dsa_transl_fetch_multi_task_nodes(struct acctest_context *ctx)
+{
+	struct task_node *tsk_node = ctx->multi_task_node;
+	int ret = ACCTEST_STATUS_OK;
+
+	while (tsk_node) {
+		tsk_node->tsk->dflags = IDXD_OP_FLAG_CRAV | IDXD_OP_FLAG_RCR;
+		if ((tsk_node->tsk->test_flags & TEST_FLAGS_BOF) && ctx->bof)
+			tsk_node->tsk->dflags |= IDXD_OP_FLAG_BOF;
+
+		dsa_prep_transl_fetch(tsk_node->tsk);
+		tsk_node = tsk_node->next;
+	}
+
+	info("Submitted all transl fetch jobs\n");
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		acctest_desc_submit(ctx, tsk_node->tsk->desc);
+		tsk_node = tsk_node->next;
+	}
+
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		ret = dsa_wait_transl_fetch(ctx, tsk_node->tsk);
+		if (ret != ACCTEST_STATUS_OK)
+			info("Desc: %p failed with ret: %d\n",
+			     tsk_node->tsk->desc, tsk_node->tsk->comp->status);
+		tsk_node = tsk_node->next;
+	}
+
+	return ret;
+}
+
 int dsa_wait_cr_delta(struct acctest_context *ctx, struct task *tsk)
 {
 	struct hw_desc *desc = tsk->desc;
@@ -1484,6 +1588,10 @@ int task_result_verify(struct task *tsk, int mismatch_expected)
 
 		rc = task_result_verify_dif_tags(tsk, tsk->xfer_size - 8 * tsk->blks);
 		return rc;
+	case DSA_OPCODE_DIX_GEN:
+		rc = task_result_verify_dif(tsk, tsk->xfer_size, mismatch_expected);
+		if (rc != ACCTEST_STATUS_OK)
+			return rc;
 	}
 
 	info("test with op %d passed\n", tsk->opcode);
@@ -1606,7 +1714,7 @@ int task_result_verify_dualcast(struct task *tsk, int mismatch_expected)
 int task_result_verify_ap_delta(struct task *tsk, int mismatch_expected)
 {
 	int rc;
-	int data_size = (tsk->comp->status == DSA_COMP_SUCCESS) ?
+	unsigned int data_size = (tsk->comp->status == DSA_COMP_SUCCESS) ?
 			 tsk->desc->xfer_size : tsk->comp->bytes_completed;
 
 	rc = memcmp((unsigned char *)tsk->desc->dst_addr,
@@ -1684,7 +1792,7 @@ int task_result_verify_crcgen(struct task *tsk, int mismatch_expected)
 {
 	unsigned int expected_crc = 0x0;
 	unsigned int seed = 0;
-	int data_size = (tsk->comp->status == DSA_COMP_SUCCESS) ?
+	unsigned int data_size = (tsk->comp->status == DSA_COMP_SUCCESS) ?
 			 tsk->desc->xfer_size : tsk->comp->bytes_completed;
 
 	if (tsk->dflags & READ_CRC_SEED)
@@ -1721,7 +1829,7 @@ int task_result_verify_crc_copy(struct task *tsk, int mismatch_expected)
 	int rc;
 	unsigned int seed = 0;
 	unsigned int expected_crc = 0x0;
-	int data_size = (tsk->comp->status == DSA_COMP_SUCCESS) ?
+	unsigned int data_size = (tsk->comp->status == DSA_COMP_SUCCESS) ?
 			 tsk->desc->xfer_size : tsk->comp->bytes_completed;
 
 	if (tsk->dflags & READ_CRC_SEED)
@@ -2066,26 +2174,79 @@ int task_result_verify_dif_tags(struct task *tsk, unsigned long xfer_size)
 	return ACCTEST_STATUS_OK;
 }
 
-int batch_result_verify(struct batch_task *btsk, int bof)
+int batch_result_verify(struct batch_task *btsk, int bof, int cpfault)
 {
 	uint8_t core_stat, sub_stat;
 	int i, rc;
 	struct task *tsk;
+	struct evl_desc_list *edl;
+	unsigned short nr_desc;
+
+	edl = btsk->edl;
+
+	if (edl && edl->bdi.bc_wr_fail)
+		return 0;
 
 	core_stat = stat_val(btsk->core_task->comp->status);
-	if (core_stat == DSA_COMP_SUCCESS) {
-		info("core task success, checking sub-tasks\n");
-	} else if (!bof && core_stat == DSA_COMP_BATCH_FAIL) {
-		info("partial complete with NBOF, checking sub-tasks\n");
+	nr_desc = btsk->task_num;
+
+	if (edl) {
+		unsigned char res;
+		struct batch_desc_info *bdi;
+
+		res = btsk->core_task->comp->result;
+		nr_desc = btsk->core_task->comp->descs_completed;
+		bdi = &edl->bdi;
+		printf("res 0x%x core_stat 0x%x nr_desc %d\n",
+		       res, core_stat, nr_desc);
+		if (res != bdi->result)
+			err("core result (0x%x) expected (0x%x)\n",
+			    res, bdi->result);
+		if (core_stat != bdi->status)
+			err("core status (0x%x) expected (0x%x)\n",
+			    core_stat, bdi->status);
+		if (res && bdi->desc_completed != nr_desc)
+			err("core descs completed (0x%x) expected (0x%x)\n",
+			    nr_desc, bdi->desc_completed);
 	} else {
-		err("batch core task failed with status %d\n", core_stat);
-		return ACCTEST_STATUS_FAIL;
+		if (core_stat == DSA_COMP_SUCCESS) {
+			info("core task success, chekcing sub-tasks\n");
+		} else if (core_stat == DSA_COMP_BATCH_PAGE_FAULT) {
+			info("batch desc list page fault\n");
+		} else if (core_stat == DSA_COMP_BATCH_FAIL) {
+			if (!bof)
+				info("partial compl. with NBOF, checking sub-tasks\n");
+			if (cpfault)
+				info("partial compl. with CPFAULT, checking sub-tasks\n");
+		} else {
+			err("batch core task failed with status %d\n", core_stat);
+			return ACCTEST_STATUS_FAIL;
+		}
 	}
 
-	for (i = 0; i < btsk->task_num; i++) {
+	rc = ACCTEST_STATUS_OK;
+
+	for (i = 0; i < nr_desc; i++) {
 		tsk = &btsk->sub_tasks[i];
+
+		if (edl && edl->di[i].cp_wr_fail)
+			continue;
+
 		sub_stat = stat_val(tsk->comp->status);
 
+		if (edl && edl->di[i].desc_fault) {
+			if (sub_stat != DSA_COMP_PAGE_FAULT_NOBOF) {
+				err("%d: expected status %u got status %u\n", i,
+				    DSA_COMP_PAGE_FAULT_NOBOF, sub_stat);
+			}
+			continue;
+		}
+
+		if (sub_stat == DSA_COMP_CRA_XLAT) {
+			dbg("Compl. Rec. PF in sub-task[%d], consider as passed\n", i);
+			continue;
+		}
+
 		if (!bof && sub_stat == DSA_COMP_PAGE_FAULT_NOBOF) {
 			dbg("PF in sub-task[%d], consider as passed\n", i);
 		} else if (sub_stat == DSA_COMP_SUCCESS) {
